% This script runs the Contrast approach to Task PLSC version of Symmetric PLS. This is the
% same example as described in the paper titled: Partial Least Squares
% (PLS) Methods for Neuroimaging: A Tutorial and Review

clear
clc

%% Load data matrix
%  For the mini-example, we have chosen a hypothetical neuroimaging
%  experiment that compares two clinical populations: Alzheimer's Disease (AD),
%  Parkinson's Disease (PD) and and an age-matched normal control group
%  (NC). Each participant is scanned once using PET imaging. Voxel activity
%  values from the PET scans are collected separately for each participant
%  and stored in matrix X. The scans are co-registered onto a common space.
%  The rows of X are participants AD1, AD2, AD3, PD1, PD2, PD3, NC1, NC2
%  and NC3. 
%  **Change this for your data**

X   =  [2     5     6     1     9     1     7     6     2     1     7     3
        4     1     5     8     8     7     2     8     6     4     8     2
        5     8     7     3     7     1     7     4     5     1     4     3
        3     3     7     6     1     1    10     2     2     1     7     4
        2     3     8     7     1     6     9     1     8     8     1     6
        1     7     3     1     1     3     1     8     1     3     9     5
        9     0     7     1     8     7     4     2     3     6     2     7
        8     0     6     5     9     7     4     4     2    10     3     8
        7     7     4     5     7     6     7     6     5     4     8     8]
    
% The data matrix is actually made of 3 sub-matrices: 
% X1 (AD), X2 (PD), X3 (NC). Each sub-matrix has 3 rows and 12 columns.
%  **Change this for your data**

X1 = X(1:3,:)
X2 = X(4:6,:)
X3 = X(7:9,:)


%% Load Contasts
%  For the mini-example, the first contrast differentiates NC from the two 
%  clinical groups and the second contrast difefrentiates AD from PD. The
%  contrasts are mean-centered (by defnition) and normalized such that the
%  sum of the squared coeffients is 1. The contrasts are stored in Matrix
%  Y.
%  **Change this for your data**

Y = [-1 -1
     -1 -1
     -1 -1
     -1  1
     -1  1
     -1  1
      2  0
      2  0
      2  0];

Y = zscore(Y)./sqrt((size(Y,1)-1))

Check = sum(Y.^2)
%% Centre and normalize Matrix X
%  Matrix X is centered and normalized by the degrees of freedom i.e., # of
%  row (observations - 1)

%Xcen = (X-repmat(mean(X),9,1))./8

Xcen = zscore(X)./sqrt((size(Y,1)-1));

Check = sum(Xcen.^2)

%% Compute correlation matrix R between Y and pre-processed Y
%  The correlation between brain activity (Matrix X) and experimental
%  design (Matrix Y) is stored in Matrix R which is now the input to the
%  SVD

R= Y'*Xcen

%% Singular Value Decomposition
%  The corrleation matrix R is decomposed into two sets of singular
%  vectors and one set of singular values. In the neuroimaging context, the
%  singular vectors are known as saliences

[U,Delta,V] = paq(R)

% Where, U are the saliences for design, V are the saliences for brain
% activity and Delta is the diagonal matrix of singular values that
% describe the covariance between each pair of singular vales for behavior
% and brain activity

%% Latent Variables
%  The latent variables for brain activity are calculated as the product of
%  the brain activity and the brain saliences (V)

Lx = Xcen*V

%  Latent variables for the brain are centered

Lx= Lx - repmat(mean(Lx),size(Lx,1),1);

%  The latent variables for the design are calculated as the product of
%  contrasts and the design saliences (U)

Ly = Y*U

%% Plots
% The latent variables for the brain (known as brain scores)
% are plotted against each other 
% **Set axes and labels for your data**
axis1 = 1;
axis2 = 2;

figure()
crosswire(min(Lx(:,axis1)),max(Lx(:,axis1)),min(Lx(:,axis2)),max(Lx(:,axis2)));
plot(Lx(:,axis1),Lx(:,axis2), 'ok','MarkerSize', 8,'MarkerFaceColor','b');
text(Lx(:,axis1),Lx(:,axis2),['   AD1';'   AD2';'   AD3';'   PD1';'   PD2';'   PD3';'   NC1';'   NC2';'   NC3']) 
xlabel('First Brain Score (Latent Variable 1)')
ylabel('Second Brain Score (Latent Variable 2)')
title('First and Second Brain Scores')

% The latent variables for the design (known as design scores)
% are plotted against each other

figure()
crosswire(min(Ly(:,axis1)),max(Ly(:,axis1)),min(Ly(:,axis2)),max(Ly(:,axis2)));
plot(Ly(:,axis1),Ly(:,axis2), 'ok','MarkerSize', 8,'MarkerFaceColor','b');
text(Ly(:,axis1),Ly(:,axis2),['   AD1';'   AD2';'   AD3';'   PD1';'   PD2';'   PD3';'   NC1';'   NC2';'   NC3']) 
xlabel('First Design Score (Latent Variable 1)')
ylabel('Second Design Score (Latent Variable 2)')
title('First and Second Design Scores')



