% This script runs the PLSR analysis. This is the
% same example as described in the paper titled: Partial Least Squares
% (PLS) Methods for Neuroimaging: A Tutorial and Review

clear
clc

%% Load data matrix
%  For the mini-example, we have chosen a hypothetical neuroimaging
%  experiment that compares two clinical populations: Alzheimer's Disease (AD),
%  Parkinson's Disease (PD) and and an age-matched normal control group
%  (NC). Each participant is scanned once using PET imaging. Voxel activity
%  values from the PET scans are collected separately for each participant
%  and stored in matrix X. The scans are co-registered onto a common space.
%  The rows of X are participants AD1, AD2, AD3, PD1, PD2, PD3, NC1, NC2
%  and NC3. 
%  **Change this for your data**

Data =[2     5     6     1     9     1     7     6     2     1     7     3
       4     1     5     8     8     7     2     8     6     4     8     2
       5     8     7     3     7     1     7     4     5     1     4     3
       3     3     7     6     1     1    10     2     2     1     7     4
       2     3     8     7     1     6     9     1     8     8     1     6
       1     7     3     1     1     3     1     8     1     3     9     5
       9     0     7     1     8     7     4     2     3     6     2     7
       8     0     6     5     9     7     4     4     2    10     3     8
       7     7     4     5     7     6     7     6     5     4     8     8];
   
%% Load Behavioral Variables
%  For the mini-example, the participants underwent behavioral testing
%  using a free-recall memory test for words measuring the number of words
%  correctly recalled and reaction time. These behavioral measures are
%  stored in Matrix Y, where the rows are the same participants as in
%  Matrix X and the columns are the Words Recalled and Reaction Time
%  respectively.  
%  **Change this for your data**
   
Behavior = [15   600
            19   520
            18   545
            22   426
            21   404
            23   411
            29   326
            30   309
            30   303];
 
 
%Refer to Herv Abdi's PLS function (PLS_jack_svds)
%Also refer to Abdi, H. (2010). Partial least square regression, projection
%on latent structure regression, PLS-Regression. Wiley Interdisciplinary
%Reviews: Computational Statistics, 2, 97-106. 

[T,P,W,Wstar,U,b,C,B_pls,Bpls_star,Xori_rec,...
    Yori_rec,Yjack,R2_X,R2_Y,RESS,PRESS,...,
    Q2,r2_random,rv_random,...
    Yjack4Press,Yhat4Res_rec]=PLS_jack_svds(Data,Behavior);
%% PLOTS
%  NOTE: The axes may be rotated when you run this example script (as
%  compared to the figures in the paper)

%  Plot for Latent Variables of X (T).
%  C are weights for Y (behavior).
%  ** Set axes and labes for your data**

axis1 = 1;
axis2 = 2;


figure()
crosswire(min([T(:,axis1);C(:,axis1)]),max([T(:,axis1);C(:,1)]),...
    min([T(:,axis2);C(:,axis2)]),max([T(:,axis2);C(:,axis2)]));
axis tight
axis square
plot(T(:,axis1),T(:,axis2), 'ok','MarkerSize', 8,'MarkerFaceColor','b');
text(T(:,axis1),T(:,axis2),['   AD1';'   AD2';'   AD3';'   PD1';'   PD2';'   PD3';'   NC1';'   NC2';'   NC3']) 
plot(C(:,axis1),C(:,axis2),'or','MarkerSize', 8,'MarkerFaceColor','r');
text(C(:,axis1),C(:,axis2),char(['  Words Recalled';'   Reaction Time']))
xlabel('First Latent Variable for X')
ylabel('Second Latent Variable for X')
title('Latent Variables 1 and 2 for X')


%   Plot for Latent Variables of Y (U)
%   C are weights for Y (behavior).

figure()
crosswire(min(U(:,axis1)),max(U(:,axis1)),min(U(:,axis2)),max(U(:,axis2)));
axis tight
axis square
plot(U(:,axis1),U(:,axis2), 'ok','MarkerSize', 8, 'MarkerFaceColor','b'); 
text(U(:,axis1),U(:,axis2),['   AD1';'   AD2';'   AD3';'   PD1';'   PD2';'   PD3';'   NC1';'   NC2';'   NC3'])  
plot(C(:,axis1),C(:,axis2),'or','MarkerSize', 8,'MarkerFaceColor','r');
text(C(:,axis1),C(:,axis2),char(['  Words Recalled';'   Reaction Time']))
xlabel('First Latent Variable for Y')
ylabel('Second Latent Variable for Y')
title('Latent Variables 1 and 2 for Y')
