% This script runs the Multi-Table version of Symmetric PLS. This is the
% same example as described in the paper titled: Partial Least Squares
% (PLS) Methods for Neuroimaging: A Tutorial and Review

clear
clc

%% Load data matrix
%  For the mini-example, we have chosen a hypothetical neuroimaging
%  experiment that compares two clinical populations: Alzheimer's Disease (AD),
%  Parkinson's Disease (PD) and and an age-matched normal control group
%  (NC). Each participant is scanned once using PET imaging. Voxel activity
%  values from the PET scans are collected separately for each participant
%  and stored in matrix X. The scans are co-registered onto a common space.
%  The rows of X are participants AD1, AD2, AD3, PD1, PD2, PD3, NC1, NC2
%  and NC3. 
%  **Change this for your data**

X   =  [2     5     6     1     9     1     7     6     2     1     7     3
        4     1     5     8     8     7     2     8     6     4     8     2
        5     8     7     3     7     1     7     4     5     1     4     3
        3     3     7     6     1     1    10     2     2     1     7     4
        2     3     8     7     1     6     9     1     8     8     1     6
        1     7     3     1     1     3     1     8     1     3     9     5
        9     0     7     1     8     7     4     2     3     6     2     7
        8     0     6     5     9     7     4     4     2    10     3     8
        7     7     4     5     7     6     7     6     5     4     8     8]

[rx, ry] = size(X);
    
% The data matrix is actually made of 3 sub-matrices: 
% X1 (AD), X2 (PD), X3 (NC). Each sub-matrix has 3 rows and 12 columns. 
%  **Change this for your data**

X1 = X(1:3,:);
X2 = X(4:6,:);
X3 = X(7:9,:);

%% Different Y matrices
%  Multi-table PLSC analyzes the relationship between brain activity and
%  multiple Y matrices. In our mini-example we chose to use a matrix of
%  orthonormal contrasts as one Y matrix and Seed voxel activity as a
%  second Y matrix. 

%% Load Contasts
%  For the mini-example, the first contrast differentiates NC from the two 
%  clinical groups and the second contrast difefrentiates AD from PD. The
%  contrasts are mean-centered (by defnition) and normalized such that the
%  sum of the squared coeffients is 1. The contrasts are stored in Matrix
%  Y.
%  **Change this for your data**

           
Y = [-1 -1
     -1 -1
     -1 -1
     -1  1
     -1  1
     -1  1
      2  0
      2  0
      2  0]
  

[ry,cy] = size(Y);
  
%Ynorm = normaliz(Y) % This uses Herv Abdi's normaliz function


% Alternatively we can compute the Z-score transform of the Y matrix and
% the ndivide each element by the square root of the number of
% observations-1. This is essentially what the normaliz function does as
% well.

Y_contrast = zscore(Y)./sqrt(ry-1)


% We check to see if the sum of squares of all elements of each column of Y
% is equal to 1

Check = sum(Y.^2)
          
           
%  Matrix X is centered and normalized the same as Matrix Y

X_norm = zscore(X)./sqrt(rx-1)

% We check to see if the sum of squares of all elements of each column of X
% is equal to 1

Check = sum(X_norm.^2)

%  The correlation between brain activity (Matrix X) and experimental
%  design (Matrix Y) is stored in Matrix R1 

R1= Y_contrast'*X_norm

%% Load Seed voxel activity
%  For the mini-example, we chose the first and last column of X as the
%  seed voxels
%  **Change this for your data**

Y_seed = [2     3
          4     2
          5     3
          3     4
          2     6
          1     5
          9     7
          8     8
          7     8]   

% The seed matrix is actually made of 3 sub-matrices: 
% Y1 (AD), Y2 (PD), Y3 (NC). Each sub-matrix has 3 rows and 2 columns.
%  **Change this for your data**


Y1_seed = Y_seed(1:3,:);
Y2_seed = Y_seed(4:6,:);
Y3_seed = Y_seed(7:9,:);

% Each sub-matrix of X and Y are Z-score transformed separately

%  Each sub-matrix of X and Y are normalized separately: Each sub-matrix is
%  normalized such that the sum of squares of each element in each
%  sub-matrix is 1. One way to do this is to first z-score transform the
%  sub matrices and then divide each element by the square root of n-1 (the
%  number of observations - 1). We use n-1 because by default the function
%  z-score uses n-1.


z1=zscore(Y1_seed)./sqrt(size(Y1_seed,1)-1);
z2=zscore(Y2_seed)./sqrt(size(Y2_seed,1)-1);
z3=zscore(Y3_seed)./sqrt(size(Y3_seed,1)-1);

Check1  = sum(z1.^2)
Check2  = sum(z2.^2)
Check3  = sum(z3.^2)

% The data matrix is normalized by condition 

d1=zscore(X1)./sqrt(size(X1,1)-1);
d2=zscore(X2)./sqrt(size(X1,1)-1);
d3=zscore(X3)./sqrt(size(X1,1)-1);

Check1  = sum(d1.^2)
Check2  = sum(d2.^2)
Check3  = sum(d3.^2)

%  Compute correlation matrix R between pairs of sub-matrices of X and Y
%  This is done by multiplying corresponding sub-matrices of X and Y (e.g.,
%  X1 and Y1, X2 and Y2 and so on). The same results can be arrived if we centre and
%  normalize the sub-matrices instead of transforming them into Z-scores.
%  However, care must be taken to see that there are no NANs.

S1=(z1'*d1);
S2=(z2'*d2);
S3=(z3'*d3);

%  Each correlation matrix is concantenated by column to form a combined
%  correlation matrix R2

R2=[S1;S2;S3]

% R1 and R2 are now concatenated by row to form matrix R which is the input
% to the SVD

R=[R1;R2]

%% Singular Value Decomposition (SVD)
%  The combined corrleation matrix R is decomposed into two sets of singular
%  vectors and one set of singular values. In the neuroimaging context, the
%  singular vectors are known as saliences
%  **Change indices for your data**

[U,Delta,V] = paq(R)

U1 = U(1:2,:);

U2 = U(3:8,:);

% Where, U are the saliences for seed voxels, V are the saliences for brain
% activity and Delta is the diagonal matrix of singular values that
% describe the covariance between each pair of singular vales for behavior
% and brain activity

%  The latent variables for the design are calculated as the product of
%  contrasts and the design saliences (The first two rows of U)

Ly_cont = Y_contrast*U1


% Because matrix Y contains a set of contrasts, each participant in a
% particular group are coded with the same contrast coefficient. When this
% matrix is multiplied with U1, we get the Latent Variables for the design
% in which each participant in each group has the same values for each
% latent variable.

% Therefore the plot will contain the same point for the whole group. In
% order to depict the results as a bar plot, we use the average value for
% each experimental group for each latent variable.

% Here we depict only the first latent variable. In this case the latent
% variable has 9 rows (three per group). We first isolate the mean value
% for each group

%  **Change indices for your data**

barmat1_b=[mean(Ly_cont(1:3,1)) mean(Ly_cont(4:6,1)) mean(Ly_cont(7:9,1))];

% We then plot these 3 values as a bar plot to show group differences 

figure()
colormap(cool)
bar(barmat1_b)
xlabel('Experimental Groups')
ylabel('First Design Score (Latent variable 1)')
title('First Design Score for all three experimental groups')


% The last 6 rows of U depict the groupwise saliences for each of the seed
% voxels. 

% Rows 3,4 represent seed 1 and seed 2 for the AD group
% Rows 5,6 represent seed 1 and seed 2 for the PD group
% Rows 7,8 represent seed 1 and seed 2 for the NC group

% We first isolate the first salience (i.e. U2(:,1)) and arrange the
% saliences according to the experimental groups.

%  **Change indices for your data**

barmat1_s=[U2(1:2,1) U2(3:4,1) U2(5:6,1)];

% Therefore the first two row of barmat1_s represents the salience values
% for seed 1 for AD, PD and NC. The second row of barmat1_s represents the
% salience values for seed 2 for AD, PD and NC.

% These two rows are plotted separately to show the group dfferences for
% each seed as depicted by the first salience.

%  **Change indices for your data**

figure()
colormap(cool)

bar(barmat1_s)
xlabel('Representative Seeds')
ylabel('First Seed Salience')
title('First Seed Salience for all three experimental groups')



