% This script runs the Seed PLSC version os Symmetric PLS. This is the
% same example as described in the paper titled: Partial Least Squares
% (PLS) Methods for Neuroimaging: A Tutorial and Review
clear
clc

%% Load data matrix
%  For the mini-example, we have chosen a hypothetical neuroimaging
%  experiment that compares two clinical populations: Alzheimer's Disease (AD),
%  Parkinson's Disease (PD) and and an age-matched normal control group
%  (NC). Each participant is scanned once using PET imaging. Voxel activity
%  values from the PET scans are collected separately for each participant
%  and stored in matrix X. The scans are co-registered onto a common space.
%  The rows of X are participants AD1, AD2, AD3, PD1, PD2, PD3, NC1, NC2
%  and NC3. 
%  **Change this for your data**

X   =  [2     5     6     1     9     1     7     6     2     1     7     3
        4     1     5     8     8     7     2     8     6     4     8     2
        5     8     7     3     7     1     7     4     5     1     4     3
        3     3     7     6     1     1    10     2     2     1     7     4
        2     3     8     7     1     6     9     1     8     8     1     6
        1     7     3     1     1     3     1     8     1     3     9     5
        9     0     7     1     8     7     4     2     3     6     2     7
        8     0     6     5     9     7     4     4     2    10     3     8
        7     7     4     5     7     6     7     6     5     4     8     8]
    
% The data matrix is actually made of 3 sub-matrices: 
% X1 (AD), X2 (PD), X3 (NC). Each sub-matrix has 3 rows and 12 columns. 

%% Load Seed voxel activity
%  For the mini-example, we chose the first and last column of X as the
%  seed voxels
%  **Change this for your data**

Y = [2     3
     4     2
     5     3
     3     4
     2     6
     1     5
     9     7
     8     8
     7     8]   
%  The seed voxels are removed from matrix X so that now matrix X has 12-2
%  = 10 columns
%  **Change this for your data**

X = [5     6     1     9     1     7     6     2     1     7
     1     5     8     8     7     2     8     6     4     8
     8     7     3     7     1     7     4     5     1     4
     3     7     6     1     1    10     2     2     1     7
     3     8     7     1     6     9     1     8     8     1
     7     3     1     1     3     1     8     1     3     9
     0     7     1     8     7     4     2     3     6     2
     0     6     5     9     7     4     4     2    10     3
     7     4     5     7     6     7     6     5     4     8]
 
X1 = X(1:3,:)
X2 = X(4:6,:)
X3 = X(7:9,:)
 
% The seed matrix is actually made of 3 sub-matrices: 
% Y1 (AD), Y2 (PD), Y3 (NC). Each sub-matrix has 3 rows and 2 columns.
%  **Change this for your data**

Y1 = Y(1:3,:)
Y2 = Y(4:6,:)
Y3 = Y(7:9,:)


%% Normalize both Matrix X and Matrix Y by sub-matrix
%  Each sub-matrix of X and Y are normalized separately: Each sub-matrix is
%  normalized such that the sum of squares of each element in each
%  sub-matrix is 1. One way to do this is to first z-score transform the
%  sub matrices and then divide each element by the square root of n-1 (the
%  number of observations - 1). We use n-1 because by default the function
%  z-score uses n-1.

%  **Change this for your data**
n_g1_obs = 3;
n_g2_obs = 3;
n_g3_obs = 3;
        
        
Xz1=zscore(X1)./sqrt(n_g1_obs-1);
Xz2=zscore(X2)./sqrt(n_g2_obs-1);
Xz3=zscore(X3)./sqrt(n_g3_obs-1);

Check = [sum(Xz1.^2);sum(Xz2.^2);sum(Xz3.^2)]

Xnorm_Seed = [Xz1;Xz2;Xz3]


Yz1=zscore(Y1)./sqrt(n_g1_obs-1);
Yz2=zscore(Y2)./sqrt(n_g2_obs-1);
Yz3=zscore(Y3)./sqrt(n_g3_obs-1);

Check = [sum(Yz1.^2);sum(Yz2.^2);sum(Yz3.^2)]


Ynorm_Seed = [Yz1;Yz2;Yz3]
        
%% Compute correlation matrix R between pairs of sub-matrices of X and Y
%  This is done by multiplying corresponding sub-matrices of X and Y (e.g.,
%  X1 and Y1, X2 and Y2 and so on) and dividing all elements of the
%  resulting matrix by the respective degrees of freedom (i.e., 3-1 = 2, in
%  this example). The same results can be arrived if we centre and
%  normalize the sub-matrices instead of transforming them into Z-scores.
%  However, care must be taken to see that there are no NANs.

R1=(Yz1'*Xz1);
R2=(Yz2'*Xz2);
R3=(Yz3'*Xz3);

%  Each correlation matrix is concantenated by column to form a combined
%  correlation matrix R that becomes the input to the SVD

R=[R1;R2;R3]

%% Singular Value Decomposition (SVD)
%  The combined corrleation matrix R is decomposed into two sets of singular
%  vectors and one set of singular values. In the neuroimaging context, the
%  singular vectors are known as saliences

[U,Delta,V] = paq(R)

% Where, U are the saliences for seed voxels, V are the saliences for brain
% activity and Delta is the diagonal matrix of singular values that
% describe the covariance between each pair of singular vales for behavior
% and brain activity

%% Latent Variables
%  The latent variables for brain activity are calculated as the product of
%  the brain activity and the brain saliences (V)

Lx = X*V

%% Plots
% The brain saliences can be projected on a glass brain to show the brain
% regions that are correlated with the behavioral measures. The Seed
% saliences can be interpreted as a type of contrast and are plotted as bar
% graphs by condition. 


% Rows 1 and 2 of Matrix U describe Seed1 and Seed2
% for the AD group. Rows 3 and 4 describe Seed1 and Seed2
% for the PD group and Rows 5 and 6 descibe the same for the NC group. Each
% column of U represents a salience. Each salience is plotted separately.
% Only the first two saliences are plotted here.

%  **Change indices for your data**

barmat1=[U(1:2,1) U(3:4,1) U(5:6,1)];
barmat2=[U(1:2,2) U(3:4,2) U(5:6,2)];

figure()
colormap(cool)
bar(barmat1)
xlabel('Seed 1 and Seed 2 for all three groups')
ylabel('First Seed Salience')
title('First Seed Salience for all three experimental groups')


figure()
colormap(cool)
bar(barmat2)
xlabel('Seed 1 and Seed 2 for all three groups')
ylabel('Second Seed Salience')
title('Second Seed Salience for all three experimental groups')




